@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
 
 #include "editortoolbar.h"
 
@@ -31,10 +9,10 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/editormanager_p.h>
 #include <coreplugin/editormanager/ieditor.h>
-#include <coreplugin/fileiconprovider.h>
 #include <coreplugin/icore.h>
 
-#include <utils/fileutils.h>
+#include <utils/filepath.h>
+#include <utils/fsengine/fileiconprovider.h>
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/utilsicons.h>
@@ -119,7 +97,7 @@ EditorToolBar::EditorToolBar(QWidget *parent) :
         Utils::StyledBar(parent), d(new EditorToolBarPrivate(parent, this))
 {
     auto toolBarLayout = new QHBoxLayout(this);
-    toolBarLayout->setMargin(0);
+    toolBarLayout->setContentsMargins(0, 0, 0, 0);
     toolBarLayout->setSpacing(0);
     toolBarLayout->addWidget(d->m_defaultToolBar);
     d->m_toolBarPlaceholder->setLayout(toolBarLayout);
@@ -128,7 +106,6 @@ EditorToolBar::EditorToolBar(QWidget *parent) :
     d->m_defaultToolBar->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     d->m_activeToolBar = d->m_defaultToolBar;
 
-    d->m_lockButton->setAutoRaise(true);
     d->m_lockButton->setEnabled(false);
 
     d->m_dragHandle->setProperty("noArrow", true);
@@ -142,6 +119,7 @@ EditorToolBar::EditorToolBar(QWidget *parent) :
 
     d->m_editorList->setProperty("hideicon", true);
     d->m_editorList->setProperty("notelideasterisk", true);
+    d->m_editorList->setProperty("elidemode", Qt::ElideMiddle);
     d->m_editorList->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
     d->m_editorList->setMinimumContentsLength(20);
     d->m_editorList->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLengthWithIcon);
@@ -149,7 +127,6 @@ EditorToolBar::EditorToolBar(QWidget *parent) :
     d->m_editorList->setMaxVisibleItems(40);
     d->m_editorList->setContextMenuPolicy(Qt::CustomContextMenu);
 
-    d->m_closeEditorButton->setAutoRaise(true);
     d->m_closeEditorButton->setIcon(Utils::Icons::CLOSE_TOOLBAR.icon());
     d->m_closeEditorButton->setEnabled(false);
     d->m_closeEditorButton->setProperty("showborder", true);
@@ -170,12 +147,11 @@ EditorToolBar::EditorToolBar(QWidget *parent) :
     splitMenu->addAction(d->m_splitNewWindowAction);
     d->m_splitButton->setMenu(splitMenu);
 
-    d->m_closeSplitButton->setAutoRaise(true);
     d->m_closeSplitButton->setIcon(Utils::Icons::CLOSE_SPLIT_BOTTOM.icon());
 
     auto toplayout = new QHBoxLayout(this);
     toplayout->setSpacing(0);
-    toplayout->setMargin(0);
+    toplayout->setContentsMargins(0, 0, 0, 0);
     toplayout->addWidget(d->m_backButton);
     toplayout->addWidget(d->m_forwardButton);
     toplayout->addWidget(d->m_lockButton);
@@ -190,15 +166,13 @@ EditorToolBar::EditorToolBar(QWidget *parent) :
 
     // this signal is disconnected for standalone toolbars and replaced with
     // a private slot connection
-    connect(d->m_editorList, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),
-            this, &EditorToolBar::listSelectionActivated);
-
+    connect(d->m_editorList, &QComboBox::activated, this, &EditorToolBar::listSelectionActivated);
     connect(d->m_editorList, &QComboBox::customContextMenuRequested, [this](QPoint p) {
        QMenu menu;
        fillListContextMenu(&menu);
        menu.exec(d->m_editorList->mapToGlobal(p));
     });
-    connect(d->m_dragHandleMenu, &QMenu::aboutToShow, [this]() {
+    connect(d->m_dragHandleMenu, &QMenu::aboutToShow, this, [this] {
        d->m_dragHandleMenu->clear();
        fillListContextMenu(d->m_dragHandleMenu);
     });
@@ -233,7 +207,7 @@ EditorToolBar::~EditorToolBar()
 void EditorToolBar::removeToolbarForEditor(IEditor *editor)
 {
     QTC_ASSERT(editor, return);
-    disconnect(editor->document(), &IDocument::changed, this, &EditorToolBar::checkDocumentStatus);
+    disconnect(editor->document(), &IDocument::changed, this, nullptr);
 
     QWidget *toolBar = editor->toolBar();
     if (toolBar != nullptr) {
@@ -267,7 +241,9 @@ void EditorToolBar::closeEditor()
 void EditorToolBar::addEditor(IEditor *editor)
 {
     QTC_ASSERT(editor, return);
-    connect(editor->document(), &IDocument::changed, this, &EditorToolBar::checkDocumentStatus);
+    auto document = editor->document();
+    connect(document, &IDocument::changed,
+            this, [this, document] { checkDocumentStatus(document); });
     QWidget *toolBar = editor->toolBar();
 
     if (toolBar && !d->m_isStandalone)
@@ -298,12 +274,14 @@ void EditorToolBar::setToolbarCreationFlags(ToolbarCreationFlags flags)
 {
     d->m_isStandalone = flags & FlagsStandalone;
     if (d->m_isStandalone) {
-        connect(EditorManager::instance(), &EditorManager::currentEditorChanged,
-                this, &EditorToolBar::updateEditorListSelection);
+        connect(EditorManager::instance(),
+                &EditorManager::currentEditorChanged,
+                this,
+                &EditorToolBar::setCurrentEditor);
 
-        disconnect(d->m_editorList, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),
+        disconnect(d->m_editorList, &QComboBox::activated,
                    this, &EditorToolBar::listSelectionActivated);
-        connect(d->m_editorList, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),
+        connect(d->m_editorList, &QComboBox::activated,
                 this, &EditorToolBar::changeActiveEditor);
         d->m_splitButton->setVisible(false);
         d->m_closeSplitButton->setVisible(false);
@@ -318,7 +296,7 @@ void EditorToolBar::setMenuProvider(const EditorToolBar::MenuProvider &provider)
 void EditorToolBar::setCurrentEditor(IEditor *editor)
 {
     IDocument *document = editor ? editor->document() : nullptr;
-    const Utils::optional<int> index = DocumentModel::rowOfDocument(document);
+    const std::optional<int> index = DocumentModel::rowOfDocument(document);
     if (QTC_GUARD(index))
         d->m_editorList->setCurrentIndex(*index);
 
@@ -330,15 +308,6 @@ void EditorToolBar::setCurrentEditor(IEditor *editor)
     updateDocumentStatus(document);
 }
 
-void EditorToolBar::updateEditorListSelection(IEditor *newSelection)
-{
-    if (newSelection) {
-        const Utils::optional<int> index = DocumentModel::rowOfDocument(newSelection->document());
-        if (QTC_GUARD(index))
-            d->m_editorList->setCurrentIndex(index.value());
-    }
-}
-
 void EditorToolBar::changeActiveEditor(int row)
 {
     EditorManager::activateEditorForEntry(DocumentModel::entryAtRow(row));
@@ -354,6 +323,8 @@ void EditorToolBar::fillListContextMenu(QMenu *menu)
                                              : nullptr;
         EditorManager::addSaveAndCloseEditorActions(menu, entry, editor);
         menu->addSeparator();
+        EditorManager::addPinEditorActions(menu, entry);
+        menu->addSeparator();
         EditorManager::addNativeDirAndOpenWithActions(menu, entry);
     }
 }
@@ -382,10 +353,8 @@ void EditorToolBar::updateActionShortcuts()
     d->m_closeSplitButton->setToolTip(ActionManager::command(Constants::REMOVE_CURRENT_SPLIT)->stringWithAppendedShortcut(tr("Remove Split")));
 }
 
-void EditorToolBar::checkDocumentStatus()
+void EditorToolBar::checkDocumentStatus(IDocument *document)
 {
-    auto document = qobject_cast<IDocument *>(sender());
-    QTC_ASSERT(document, return);
     DocumentModel::Entry *entry = DocumentModel::entryAtRow(
                 d->m_editorList->currentIndex());
 
@@ -425,7 +394,7 @@ void EditorToolBar::updateDocumentStatus(IDocument *document)
     if (document->filePath().isEmpty())
         d->m_dragHandle->setIcon(QIcon());
     else
-        d->m_dragHandle->setIcon(FileIconProvider::icon(document->filePath().toFileInfo()));
+        d->m_dragHandle->setIcon(Utils::FileIconProvider::icon(document->filePath()));
 
     d->m_editorList->setToolTip(document->filePath().isEmpty()
                                 ? document->displayName()
@@ -456,7 +425,7 @@ bool EditorToolBar::eventFilter(QObject *obj, QEvent *event)
                 return Utils::StyledBar::eventFilter(obj, event);
             auto drag = new QDrag(this);
             auto data = new Utils::DropMimeData;
-            data->addFile(entry->fileName().toString());
+            data->addFile(entry->fileName());
             drag->setMimeData(data);
             Qt::DropAction action = drag->exec(Qt::MoveAction | Qt::CopyAction, Qt::MoveAction);
             if (action == Qt::MoveAction)
