@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
 
 #include "jsonwizardfactory.h"
 
@@ -40,6 +18,7 @@
 #include <extensionsystem/pluginmanager.h>
 
 #include <utils/algorithm.h>
+#include <utils/environment.h>
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/stringutils.h>
@@ -321,7 +300,7 @@ std::pair<int, QStringList> JsonWizardFactory::screenSizeInfoFromPage(const QStr
     if (values.isEmpty())
         return {};
 
-    return std::make_pair(index, values);
+    return {index, values};
 }
 
 JsonWizardFactory::Page JsonWizardFactory::parsePage(const QVariant &value, QString *errorMessage)
@@ -398,13 +377,12 @@ JsonWizardFactory::Page JsonWizardFactory::parsePage(const QVariant &value, QStr
 //FIXME: loadDefaultValues() has an almost identical loop. Make the loop return the results instead of
 //internal processing and create a separate function for it. Then process the results in
 //loadDefaultValues() and loadDefaultValues()
-QList<Core::IWizardFactory *> JsonWizardFactory::createWizardFactories()
+void JsonWizardFactory::createWizardFactories()
 {
     QString errorMessage;
     QString verboseLog;
     const QString wizardFileName = QLatin1String(WIZARD_FILE);
 
-    QList <Core::IWizardFactory *> result;
     const Utils::FilePaths paths = searchPaths();
     for (const Utils::FilePath &path : paths) {
         if (path.isEmpty())
@@ -432,7 +410,7 @@ QList<Core::IWizardFactory *> JsonWizardFactory::createWizardFactories()
             const FilePath currentFile = currentDir / wizardFileName;
             if (currentFile.exists()) {
                 QJsonParseError error;
-                const QByteArray fileData = currentFile.fileContents();
+                const QByteArray fileData = currentFile.fileContents().value_or(QByteArray());
                 const QJsonDocument json = QJsonDocument::fromJson(fileData, &error);
 
                 if (error.error != QJsonParseError::NoError) {
@@ -470,13 +448,10 @@ QList<Core::IWizardFactory *> JsonWizardFactory::createWizardFactories()
                     continue;
                 }
 
-                JsonWizardFactory *factory = createWizardFactory(data, currentDir, &errorMessage);
-                if (!factory) {
-                    verboseLog.append(tr("* Failed to create: %1\n").arg(errorMessage));
-                    continue;
-                }
-
-                result << factory;
+                IWizardFactory::registerFactoryCreator([data, currentDir] {
+                    QString errorMessage;
+                    return createWizardFactory(data, currentDir, &errorMessage);
+                });
             } else {
                 FilePaths subDirs = currentDir.dirEntries(filter, sortflags);
                 if (!subDirs.isEmpty()) {
@@ -495,7 +470,6 @@ QList<Core::IWizardFactory *> JsonWizardFactory::createWizardFactories()
         Core::MessageManager::writeDisrupting(verboseLog);
     }
 
-    return result;
 }
 
 JsonWizardFactory *JsonWizardFactory::createWizardFactory(const QVariantMap &data,
@@ -506,6 +480,7 @@ JsonWizardFactory *JsonWizardFactory::createWizardFactory(const QVariantMap &dat
     if (!factory->initialize(data, baseDir, errorMessage)) {
         delete factory;
         factory = nullptr;
+        Core::MessageManager::writeDisrupting(*errorMessage);
     }
     return factory;
 }
@@ -514,7 +489,7 @@ static QStringList environmentTemplatesPaths()
 {
     QStringList paths;
 
-    QString envTempPath = QString::fromLocal8Bit(qgetenv("QTCREATOR_TEMPLATES_PATH"));
+    QString envTempPath = qtcEnvironmentVariable("QTCREATOR_TEMPLATES_PATH");
 
     if (!envTempPath.isEmpty()) {
         for (const QString &path : envTempPath
@@ -530,10 +505,15 @@ static QStringList environmentTemplatesPaths()
 
 FilePaths &JsonWizardFactory::searchPaths()
 {
-    static FilePaths m_searchPaths = {Core::ICore::userResourcePath(WIZARD_PATH),
-                                      Core::ICore::resourcePath(WIZARD_PATH)};
-    for (const QString &environmentTemplateDirName : environmentTemplatesPaths())
-        m_searchPaths << FilePath::fromString(environmentTemplateDirName);
+    static FilePaths m_searchPaths;
+    static bool searchPathsInitialized = false;
+    if (!searchPathsInitialized) {
+        searchPathsInitialized = true;
+        m_searchPaths = {Core::ICore::userResourcePath(WIZARD_PATH),
+                         Core::ICore::resourcePath(WIZARD_PATH)};
+        for (const QString &environmentTemplateDirName : environmentTemplatesPaths())
+            m_searchPaths << FilePath::fromString(environmentTemplateDirName);
+    }
 
     return m_searchPaths;
 }
@@ -624,13 +604,13 @@ Wizard *JsonWizardFactory::runWizardImpl(const FilePath &path, QWidget *parent,
     wizard->setValue(QStringLiteral("id"), id().toString());
 
     MacroExpander *expander = wizard->expander();
-    for (const JsonWizard::OptionDefinition &od : qAsConst(m_options)) {
+    for (const JsonWizard::OptionDefinition &od : std::as_const(m_options)) {
         if (od.condition(*expander))
             wizard->setValue(od.key(), od.value(*expander));
     }
 
     bool havePage = false;
-    for (const Page &data : qAsConst(m_pages)) {
+    for (const Page &data : std::as_const(m_pages)) {
         QTC_ASSERT(data.isValid(), continue);
 
         if (!JsonWizard::boolFromVariant(data.enabled, wizard->expander()))
@@ -658,7 +638,7 @@ Wizard *JsonWizardFactory::runWizardImpl(const FilePath &path, QWidget *parent,
         }
     }
 
-    for (const Generator &data : qAsConst(m_generators)) {
+    for (const Generator &data : std::as_const(m_generators)) {
         QTC_ASSERT(data.isValid(), continue);
         JsonWizardGeneratorFactory *factory = Utils::findOr(s_generatorFactories, nullptr,
                                                             [&data](JsonWizardGeneratorFactory *f) {
@@ -705,7 +685,7 @@ QString JsonWizardFactory::localizedString(const QVariant &value)
         const QString locale = languageSetting().toLower();
         QStringList locales;
         locales << locale << QLatin1String("en") << QLatin1String("C") << tmp.keys();
-        for (const QString &locale : qAsConst(locales)) {
+        for (const QString &locale : std::as_const(locales)) {
             QString result = tmp.value(locale, QString()).toString();
             if (!result.isEmpty())
                 return result;
@@ -792,12 +772,16 @@ bool JsonWizardFactory::initialize(const QVariantMap &data, const FilePath &base
     }
     setCategory(strVal);
 
-    strVal = data.value(QLatin1String(ICON_KEY)).toString();
-    const FilePath iconPath = baseDir.resolvePath(strVal);
-    if (!iconPath.exists()) {
-        *errorMessage = tr("Icon file \"%1\" not found.").arg(iconPath.toUserOutput());
-        return false;
+    FilePath iconPath;
+    if (data.contains(QLatin1String(ICON_KEY))) {
+        strVal = data.value(QLatin1String(ICON_KEY)).toString();
+        iconPath = baseDir.resolvePath(strVal);
+        if (!iconPath.exists()) {
+            *errorMessage = tr("Icon file \"%1\" not found.").arg(iconPath.toUserOutput());
+            return false;
+        }
     }
+
     const QString iconText = data.value(QLatin1String(ICON_TEXT_KEY)).toString();
     const bool iconIsThemed = data.value(QLatin1String(ICON_KIND_KEY)).toString()
             .compare("Themed", Qt::CaseInsensitive) == 0;
@@ -855,7 +839,7 @@ bool JsonWizardFactory::initialize(const QVariantMap &data, const FilePath &base
         return false;
     }
 
-    for (const QVariant &v : qAsConst(list)) {
+    for (const QVariant &v : std::as_const(list)) {
         Generator gen = parseGenerator(v, errorMessage);
         if (gen.isValid())
             m_generators.append(gen);
@@ -870,7 +854,7 @@ bool JsonWizardFactory::initialize(const QVariantMap &data, const FilePath &base
         return false;
     }
 
-    for (const QVariant &v : qAsConst(list)) {
+    for (const QVariant &v : std::as_const(list)) {
         Page p = parsePage(v, errorMessage);
         if (p.isValid())
             m_pages.append(p);
